@@ -203,9 +203,11 @@ class File(object):
         return [File.bcopy(fname, dest_dir, overwrite, verbose=True) for fname in filenames]
 
     @classmethod
-    def append_ext(cls, filepath, append_key, directory=None, fext=None):
+    def append_ext(cls, filepath, append_key, directory=None, fext=None, abs=True):
         filename = File.base(filepath)
         afilename = filename.replace('.', f'_{append_key}.')
+        if abs:
+            directory = File.getdir(filepath)
         if fext:
             afilename = afilename.rsplit('.', 1)[0] + f'.{fext}'
         if directory:
@@ -361,7 +363,8 @@ class File(object):
                 yield File.pload(filename)
             
             elif filename.endswith('.jsonl') or filename.endswith('.jsonlines'):
-                yield File.jlg(filename)
+                iterator = File.jlg(filename)
+                yield from iterator
             
             elif filename.endswith('.json'):
                 yield File.jsonload(filename)
@@ -370,7 +373,8 @@ class File(object):
                 yield File.ptload(filename, device)
             
             elif filename.endswith('.txt'):
-                yield File.textload(filename)
+                iterator = File.textload(filename)
+                yield from iterator
             
             elif filename.endswith('.csv'):
                 try:
@@ -386,6 +390,7 @@ class File(object):
             
             else:
                 logger.info(f'Unrecognized File Extension: {filename}')
+        return
 
     
     @classmethod
@@ -633,7 +638,29 @@ class File(object):
     @classmethod
     def tsvreader(cls, f):
         return csv.DictReader(f, delimiter='\t')
+    
+    @classmethod
+    def csvwrite(cls, data, filename, mode='auto', keys=None, delimiter=','):
+        mode = mode if mode != 'auto' else File.autowrite(filename)
+        _is_dict = False
+        if isinstance(data, dict):
+            keys = keys or list(data.keys())
+            _is_dict = True
+        
+        elif isinstance(data, list):
+            if isinstance(data[0], dict):
+                _is_dict = True
+                keys = keys or list(data[0].keys())
 
+        with gfile(filename, mode) as f:
+            writer = csv.DictWriter(f, keys, delimiter=delimiter) if _is_dict else csv.writer(f, delimiter=delimiter)
+            if keys and mode != 'a':
+                writer.writeheader()
+            writer.writerows(data)
+    
+    @classmethod
+    def tsvwrite(cls, data, filename, mode='auto', keys=None):
+        return File.csvwrite(data, filename, mode, keys, delimiter='\t')
 
     # Text Lines Methods
     @classmethod
@@ -1001,11 +1028,11 @@ class File(object):
             File.copy(src_dict[key], dest_dict[key], overwrite=overwrite)
     
     @classmethod
-    def get_local(cls, filenames, directory='/content/data'):
+    def get_local(cls, filenames, directory='/content/data', ovewrite=False):
         filenames = File.fsorter(filenames)
         lpaths = []
         for fpath in filenames:
-            lpath = File.bcopy(fpath, directory, overwrite=False)
+            lpath = File.bcopy(fpath, directory, overwrite=ovewrite)
             lpaths.append(lpath)
         return lpaths
     
@@ -1038,7 +1065,12 @@ class File(object):
 
     @classmethod
     def split_file(cls, filename, split_dict={'train': 0.85, 'val': 0.15, 'test': 0.05}, output_format='jsonl', directory=None, shuffle=True):
-        items = [ex for ex in File.load(filename)]
+        iterator = File.load(filename)
+        #iterator = File.jlg(filename)
+        items = []
+        for ex in iterator:
+            items.append(ex)
+        #items = [ex for ex in iterator]
         out_fns = {k: File.append_ext(filename, k, directory) for k in list(split_dict.keys())}
         out_fns['results'] = File.append_ext(filename, 'results', directory, 'json')
         split_data = File.split_items(items, split_dict, shuffle)
